@@ -7,7 +7,7 @@ module ActiveRecord
     class PostgreSQLAdapter
       class QuotingTest < ActiveRecord::PostgreSQLTestCase
         def setup
-          @conn = ActiveRecord::Base.connection
+          @conn = ActiveRecord::Base.lease_connection
           @raise_int_wider_than_64bit = ActiveRecord.raise_int_wider_than_64bit
         end
 
@@ -58,6 +58,25 @@ module ActiveRecord
           assert_equal "\"user posts\"", @conn.quote_table_name(value)
         end
 
+        def test_quote_string
+          assert_equal "''", @conn.quote_string("'")
+        end
+
+        def test_quote_column_name
+          [@conn, @conn.class].each do |adapter|
+            assert_equal '"foo"', adapter.quote_column_name("foo")
+            assert_equal '"hel""lo"', adapter.quote_column_name(%{hel"lo})
+          end
+        end
+
+        def test_quote_table_name
+          [@conn, @conn.class].each do |adapter|
+            assert_equal '"foo"', adapter.quote_table_name("foo")
+            assert_equal '"foo"."bar"', adapter.quote_table_name("foo.bar")
+            assert_equal '"hel""lo.wol\\d"', adapter.quote_column_name('hel"lo.wol\\d')
+          end
+        end
+
         def test_raise_when_int_is_wider_than_64bit
           value = 9223372036854775807 + 1
           assert_raise ActiveRecord::ConnectionAdapters::PostgreSQL::Quoting::IntegerOutOf64BitRange do
