@@ -154,6 +154,14 @@ module ActiveRecord
       end
     end
 
+    class DropTableMigration < SilentMigration
+      def change
+        drop_table("horses") do |t|
+          t.string :name
+        end
+      end
+    end
+
     class LegacyMigration < ActiveRecord::Migration::Current
       def self.up
         create_table("horses") do |t|
@@ -433,6 +441,20 @@ module ActiveRecord
       end
     end
 
+    def test_migrate_revert_drop_table
+      connection = ActiveRecord::Base.lease_connection
+      migration1 = InvertibleMigration.new
+      migration1.migrate(:up)
+      assert connection.table_exists?("horses")
+
+      migration2 = DropTableMigration.new
+      migration2.migrate(:up)
+      assert_not connection.table_exists?("horses")
+
+      migration2.migrate(:down)
+      assert connection.table_exists?("horses")
+    end
+
     def test_revert_order
       block = Proc.new { |t| t.string :name }
       recorder = ActiveRecord::Migration::CommandRecorder.new(ActiveRecord::Base.lease_connection)
