@@ -15,11 +15,10 @@ module ActiveRecord
       :map, :none?, :one?, :partition, :reject, :reverse, :rotate,
       :sample, :second, :sort, :sort_by, :slice, :third, :index, :rindex,
       :to_ary, :to_set, :to_xml, :to_yaml, :join,
-      :in_groups, :in_groups_of, :to_sentence, :to_formatted_s, :to_fs, :as_json
+      :in_groups, :in_groups_of, :to_sentence, :to_formatted_s, :to_fs, :as_json,
+      :intersect?
     ]
 
-    ARRAY_DELEGATES << :intersect? if RUBY_VERSION >= "3.1.0"
-
     ARRAY_DELEGATES.each do |method|
       define_method "test_delegates_#{method}_to_Array" do
         assert_respond_to target, method
@@ -63,12 +62,13 @@ module ActiveRecord
       ActiveRecord::SpawnMethods.public_instance_methods(false) - [:spawn, :merge!] +
       ActiveRecord::QueryMethods.public_instance_methods(false).reject { |method|
         method.end_with?("=", "!", "?", "value", "values", "clause")
-      } - [:reverse_order, :arel, :extensions, :construct_join_dependency] + [
+      } - [:all, :reverse_order, :arel, :extensions, :construct_join_dependency] + [
         :any?, :many?, :none?, :one?,
         :first_or_create, :first_or_create!, :first_or_initialize,
         :find_or_create_by, :find_or_create_by!, :find_or_initialize_by,
         :create_or_find_by, :create_or_find_by!,
-        :destroy_all, :delete_all, :update_all, :touch_all, :delete_by, :destroy_by
+        :destroy, :destroy_all, :delete, :delete_all, :update_all, :touch_all, :delete_by, :destroy_by,
+        :insert, :insert_all, :insert!, :insert_all!, :upsert, :upsert_all,
       ]
 
     def test_delegate_querying_methods
@@ -90,14 +90,14 @@ module ActiveRecord
 
     test "delegation doesn't override methods defined in other relation subclasses" do
       # precondition, some methods are available on ActiveRecord::Relation subclasses
-      # but not ActiveRecord::Relation itself. Here `delete` is just an example.
-      assert_equal false, ActiveRecord::Relation.method_defined?(:delete)
-      assert_equal true, ActiveRecord::Associations::CollectionProxy.method_defined?(:delete)
+      # but not ActiveRecord::Relation itself. Here `target` is just an example.
+      assert_equal false, ActiveRecord::Relation.method_defined?(:target)
+      assert_equal true, ActiveRecord::Associations::CollectionProxy.method_defined?(:target)
 
       project = projects(:active_record)
-      original_owner = project.developers_with_callbacks.method(:delete).owner
-      Developer.all.delete(12345)
-      assert_equal original_owner, project.developers_with_callbacks.method(:delete).owner
+      original_owner = project.developers_with_callbacks.method(:target).owner
+      assert_equal :__target__, Developer.all.target
+      assert_equal original_owner, project.developers_with_callbacks.method(:target).owner
     end
   end
 end
