@@ -5,28 +5,12 @@ require "models/post"
 require "models/category"
 require "models/comment"
 require "models/other_dog"
+require "concurrent/atomic/count_down_latch"
 
 module ActiveRecord
-  module WaitForAsyncTestHelper
-    private
-      def wait_for_async_query(connection = ActiveRecord::Base.connection, timeout: 5)
-        return unless connection.async_enabled?
-
-        executor = connection.pool.async_executor
-        (timeout * 100).times do
-          return unless executor.scheduled_task_count > executor.completed_task_count
-          sleep 0.01
-        end
-
-        raise Timeout::Error, "The async executor wasn't drained after #{timeout} seconds"
-      end
-  end
-
   class LoadAsyncTest < ActiveRecord::TestCase
     include WaitForAsyncTestHelper
 
-    self.use_transactional_tests = false
-
     fixtures :posts, :comments, :categories, :categories_posts
 
     def test_scheduled?
@@ -124,9 +108,9 @@ module ActiveRecord
       wait_for_async_query
 
       assert_equal expected_records, deferred_posts.to_a
-      assert_equal Post.connection.supports_concurrent_connections?, status[:async]
+      assert_equal Post.lease_connection.supports_concurrent_connections?, status[:async]
       assert_equal Thread.current.object_id, status[:thread_id]
-      if Post.connection.supports_concurrent_connections?
+      if Post.lease_connection.supports_concurrent_connections?
         assert_instance_of Float, status[:lock_wait]
       else
         assert_nil status[:lock_wait]
@@ -151,7 +135,7 @@ module ActiveRecord
       wait_for_async_query
 
       assert_equal expected_records, deferred_posts.to_a
-      assert_equal Post.connection.supports_concurrent_connections?, status[:async]
+      assert_equal Post.lease_connection.supports_concurrent_connections?, status[:async]
     ensure
       ActiveSupport::Notifications.unsubscribe(subscriber) if subscriber
     end
@@ -174,6 +158,42 @@ module ActiveRecord
       assert_equal ["In Transaction"], posts.map(&:title).uniq
     end
 
+    def test_load_async_instrumentation_is_thread_safe
+      skip unless ActiveRecord::Base.connection.async_enabled?
+
+      begin
+        latch1 = Concurrent::CountDownLatch.new
+        latch2 = Concurrent::CountDownLatch.new
+
+        old_log = ActiveRecord::Base.connection.method(:log)
+        ActiveRecord::Base.connection.singleton_class.undef_method(:log)
+
+        ActiveRecord::Base.connection.singleton_class.define_method(:log) do |*args, **kwargs, &block|
+          unless kwargs[:async]
+            return old_log.call(*args, **kwargs, &block)
+          end
+
+          latch1.count_down
+          latch2.wait
+          old_log.call(*args, **kwargs, &block)
+        end
+
+        Post.async_count
+        latch1.wait
+
+        notification_called = false
+        ActiveSupport::Notifications.subscribed(->(*) { notification_called = true }, "sql.active_record") do
+          Post.count
+        end
+
+        assert(notification_called)
+      ensure
+        latch2.count_down
+        ActiveRecord::Base.connection.singleton_class.undef_method(:log)
+        ActiveRecord::Base.connection.singleton_class.define_method(:log, old_log)
+      end
+    end
+
     def test_eager_loading_query
       expected_records = Post.where(author_id: 1).eager_load(:comments).to_a
 
@@ -196,16 +216,16 @@ module ActiveRecord
       end
 
       assert_equal expected_records, deferred_posts.to_a
-      assert_queries(0) do
+      assert_queries_count(0) do
         deferred_posts.each(&:comments)
       end
-      assert_equal Post.connection.supports_concurrent_connections?, status[:async]
+      assert_equal Post.lease_connection.supports_concurrent_connections?, status[:async]
     ensure
       ActiveSupport::Notifications.unsubscribe(subscriber) if subscriber
     end
 
     def test_contradiction
-      assert_queries(0) do
+      assert_queries_count(0) do
         assert_equal [], Post.where(id: []).load_async.to_a
       end
 
@@ -217,6 +237,11 @@ module ActiveRecord
       assert_equal titles, Post.where(author_id: 1).load_async.pluck(:title)
     end
 
+    def test_count
+      count = Post.where(author_id: 1).count
+      assert_equal count, Post.where(author_id: 1).load_async.count
+    end
+
     def test_size
       expected_size = Post.where(author_id: 1).size
 
@@ -233,18 +258,23 @@ module ActiveRecord
       assert_predicate deferred_posts, :loaded?
     end
 
-    def test_load_async_with_query_cache
+    def test_load_async_pluck_with_query_cache
       titles = Post.where(author_id: 1).pluck(:title)
       Post.cache do
         assert_equal titles, Post.where(author_id: 1).load_async.pluck(:title)
       end
     end
+
+    def test_load_async_count_with_query_cache
+      count = Post.where(author_id: 1).count
+      Post.cache do
+        assert_equal count, Post.where(author_id: 1).load_async.count
+      end
+    end
   end
 
   class LoadAsyncNullExecutorTest < ActiveRecord::TestCase
     unless in_memory_db?
-      self.use_transactional_tests = false
-
       fixtures :posts, :comments
 
       def setup
@@ -286,7 +316,7 @@ module ActiveRecord
         deferred_posts = Post.where(author_id: 1).load_async
 
         assert_equal expected_records, deferred_posts.to_a
-        assert_not_equal Post.connection.supports_concurrent_connections?, status[:async]
+        assert_not_equal Post.lease_connection.supports_concurrent_connections?, status[:async]
       ensure
         ActiveSupport::Notifications.unsubscribe(subscriber) if subscriber
       end
@@ -322,18 +352,18 @@ module ActiveRecord
         assert_not_predicate deferred_posts, :scheduled?
 
         assert_equal expected_records, deferred_posts.to_a
-        assert_queries(0) do
+        assert_queries_count(0) do
           deferred_posts.each(&:comments)
         end
 
-        assert_predicate Post.connection, :supports_concurrent_connections?
+        assert_predicate Post.lease_connection, :supports_concurrent_connections?
         assert_not status[:async], "Expected status[:async] to be false with NullExecutor"
       ensure
         ActiveSupport::Notifications.unsubscribe(subscriber) if subscriber
       end
 
       def test_contradiction
-        assert_queries(0) do
+        assert_queries_count(0) do
           assert_equal [], Post.where(id: []).load_async.to_a
         end
 
@@ -367,8 +397,6 @@ module ActiveRecord
     unless in_memory_db?
       include WaitForAsyncTestHelper
 
-      self.use_transactional_tests = false
-
       fixtures :posts, :comments
 
       def setup
@@ -429,7 +457,7 @@ module ActiveRecord
         wait_for_async_query
 
         assert_equal expected_records, deferred_posts.to_a
-        assert_equal Post.connection.supports_concurrent_connections?, status[:async]
+        assert_equal Post.lease_connection.supports_concurrent_connections?, status[:async]
       ensure
         ActiveSupport::Notifications.unsubscribe(subscriber) if subscriber
       end
@@ -465,16 +493,16 @@ module ActiveRecord
         assert_predicate deferred_posts, :scheduled?
 
         assert_equal expected_records, deferred_posts.to_a
-        assert_queries(0) do
+        assert_queries_count(0) do
           deferred_posts.each(&:comments)
         end
-        assert_equal Post.connection.supports_concurrent_connections?, status[:async]
+        assert_equal Post.lease_connection.supports_concurrent_connections?, status[:async]
       ensure
         ActiveSupport::Notifications.unsubscribe(subscriber) if subscriber
       end
 
       def test_contradiction
-        assert_queries(0) do
+        assert_queries_count(0) do
           assert_equal [], Post.where(id: []).load_async.to_a
         end
 
@@ -508,8 +536,6 @@ module ActiveRecord
     unless in_memory_db?
       include WaitForAsyncTestHelper
 
-      self.use_transactional_tests = false
-
       fixtures :posts, :comments, :other_dogs
 
       def setup
@@ -577,8 +603,8 @@ module ActiveRecord
         assert_equal expected_records, deferred_posts.to_a
         assert_equal expected_dogs, deferred_dogs.to_a
 
-        assert_equal Post.connection.async_enabled?, status[:async]
-        assert_equal OtherDog.connection.async_enabled?, dog_status[:async]
+        assert_equal Post.lease_connection.async_enabled?, status[:async]
+        assert_equal OtherDog.lease_connection.async_enabled?, dog_status[:async]
       ensure
         ActiveSupport::Notifications.unsubscribe(subscriber) if subscriber
       end
