@@ -89,7 +89,7 @@ class StoreTest < ActiveRecord::TestCase
 
   test "updating the store will mark accessor as changed" do
     @john.color = "red"
-    assert @john.color_changed?
+    assert_predicate @john, :color_changed?
   end
 
   test "new record and no accessors changes" do
@@ -99,14 +99,14 @@ class StoreTest < ActiveRecord::TestCase
     assert_nil user.color_change
 
     user.color = "red"
-    assert user.color_changed?
+    assert_predicate user, :color_changed?
     assert_nil user.color_was
     assert_equal "red", user.color_change[1]
   end
 
   test "updating the store won't mark accessor as changed if the whole store was updated" do
     @john.settings = { color: @john.color, some: "thing" }
-    assert @john.settings_changed?
+    assert_predicate @john, :settings_changed?
     assert_not @john.color_changed?
   end
 
@@ -133,46 +133,46 @@ class StoreTest < ActiveRecord::TestCase
   test "nullifying the store mark accessor as changed" do
     color = @john.color
     @john.settings = nil
-    assert @john.color_changed?
+    assert_predicate @john, :color_changed?
     assert_equal color, @john.color_was
     assert_equal [color, nil], @john.color_change
   end
 
   test "dirty methods for suffixed accessors" do
     @john.configs[:two_factor_auth] = true
-    assert @john.two_factor_auth_configs_changed?
+    assert_predicate @john, :two_factor_auth_configs_changed?
     assert_nil @john.two_factor_auth_configs_was
     assert_equal [nil, true], @john.two_factor_auth_configs_change
   end
 
   test "dirty methods for prefixed accessors" do
     @john.spouse[:name] = "Lena"
-    assert @john.partner_name_changed?
+    assert_predicate @john, :partner_name_changed?
     assert_equal "Dallas", @john.partner_name_was
     assert_equal ["Dallas", "Lena"], @john.partner_name_change
   end
 
   test "saved changes tracking for accessors" do
     @john.spouse[:name] = "Lena"
-    assert @john.partner_name_changed?
+    assert_predicate @john, :partner_name_changed?
 
     @john.save!
     assert_not @john.partner_name_change
-    assert @john.saved_change_to_partner_name?
+    assert_predicate @john, :saved_change_to_partner_name?
     assert_equal ["Dallas", "Lena"], @john.saved_change_to_partner_name
     assert_equal "Dallas", @john.partner_name_before_last_save
   end
 
   test "saved changes tracking for accessors with json column" do
-    if current_adapter?(:Mysql2Adapter, :TrilogyAdapter) && ActiveRecord::Base.connection.mariadb?
+    if current_adapter?(:Mysql2Adapter, :TrilogyAdapter) && ActiveRecord::Base.lease_connection.mariadb?
       skip "MariaDB doesn't support JSON store_accessor"
     end
     @john.enable_friend_requests = true
-    assert @john.enable_friend_requests_changed?
+    assert_predicate @john, :enable_friend_requests_changed?
 
     @john.save!
     assert_not @john.enable_friend_requests_change
-    assert @john.saved_change_to_enable_friend_requests?
+    assert_predicate @john, :saved_change_to_enable_friend_requests?
     assert_equal [nil, true], @john.saved_change_to_enable_friend_requests
     # Make a second change to test key_before_last_save
     @john.enable_friend_requests = false
@@ -220,6 +220,14 @@ class StoreTest < ActiveRecord::TestCase
     assert_equal "blue", user.color[:jenny]
   end
 
+  test "store takes precedence when updating store and accessor" do
+    user = Admin::User.find_by_name("Jamis")
+    user.update(settings: { homepage: "rails" }, homepage: "not rails")
+
+    assert_equal "rails", user.settings[:homepage]
+    assert_equal "rails", user.homepage
+  end
+
   def test_convert_store_attributes_from_Hash_to_HashWithIndifferentAccess_saving_the_data_and_access_attributes_indifferently
     user = Admin::User.find_by_name("Jamis")
     assert_equal "symbol",  user.settings[:symbol]
@@ -359,4 +367,18 @@ class StoreTest < ActiveRecord::TestCase
   test "prefix/suffix do not affect stored attributes" do
     assert_equal [:secret_question, :two_factor_auth, :login_retry], Admin::User.stored_attributes[:configs]
   end
+
+  test "store_accessor raises an exception if the column is not either serializable or a structured type" do
+    user = Class.new(Admin::User) do
+      store_accessor :name, :color
+    end.new
+
+    assert_raises ActiveRecord::ConfigurationError do
+      user.color
+    end
+
+    assert_raises ActiveRecord::ConfigurationError do
+      user.color = "blue"
+    end
+  end
 end
