@@ -154,6 +154,12 @@ ActiveRecord::Schema.define do
     t.date :updated_on
   end
 
+  create_table :book_identifiers, id: :integer, force: true do |t|
+    t.references :book
+    t.string :id_type, null: false
+    t.string :id_value, null: false
+  end
+
   create_table :encrypted_books, id: :integer, force: true do |t|
     t.references :author
     t.string :format
@@ -265,6 +271,12 @@ ActiveRecord::Schema.define do
     t.string :author
   end
 
+  create_table :cpk_posts_tags, force: true do |t|
+    t.string :post_title
+    t.string :post_author
+    t.integer :tag_id
+  end
+
   create_table :cpk_comments, force: true do |t|
     t.string :commentable_title
     t.string :commentable_author
