@@ -39,8 +39,8 @@ class DeprecationTest < ActiveSupport::TestCase
     end
   end
 
-  test "assert_deprecated is deprecated without a deprecator" do
-    assert_deprecated(ActiveSupport.deprecator) do
+  test "assert_deprecated requires a deprecator" do
+    assert_raises(ArgumentError) do
       assert_deprecated do
         ActiveSupport::Deprecation._instance.warn
       end
@@ -53,8 +53,8 @@ class DeprecationTest < ActiveSupport::TestCase
     end
   end
 
-  test "assert_not_deprecated is deprecated without a deprecator" do
-    assert_deprecated(ActiveSupport.deprecator) do
+  test "assert_not_deprecated requires a deprecator" do
+    assert_raises(ArgumentError) do
       assert_not_deprecated { }
     end
   end
@@ -69,8 +69,8 @@ class DeprecationTest < ActiveSupport::TestCase
     assert_match "DEPRECATION WARNING:", result.last.sole
   end
 
-  test "collect_deprecations is deprecated without a deprecator" do
-    assert_deprecated(ActiveSupport.deprecator) do
+  test "collect_deprecations requires a deprecator" do
+    assert_raises(ArgumentError) do
       collect_deprecations { }
     end
   end
@@ -108,15 +108,11 @@ class DeprecationTest < ActiveSupport::TestCase
     end
   end
 
-  test "Module::deprecate without a deprecator is deprecated" do
+  test "Module::deprecate requires a deprecator" do
     klass = Class.new(Deprecatee)
-    _, deprecations = collect_deprecations(ActiveSupport.deprecator) do
+    assert_raises(ArgumentError) do
       klass.deprecate :zero
     end
-    assert_match "Module.deprecate without a deprecator is deprecated", deprecations.sole
-    assert_deprecated(/zero is deprecated/, ActiveSupport::Deprecation._instance) do
-      klass.new.zero
-    end
   end
 
   test "DeprecatedObjectProxy" do
@@ -124,8 +120,8 @@ class DeprecationTest < ActiveSupport::TestCase
     assert_deprecated(/:bomb:/, @deprecator) { deprecated_object.to_s }
   end
 
-  test "DeprecatedObjectProxy without a deprecator is deprecated" do
-    assert_deprecated(ActiveSupport.deprecator) do
+  test "DeprecatedObjectProxy requires a deprecator" do
+    assert_raises(ArgumentError) do
       ActiveSupport::Deprecation::DeprecatedObjectProxy.new(Object.new, ":bomb:")
     end
   end
@@ -192,11 +188,22 @@ class DeprecationTest < ActiveSupport::TestCase
     assert_match "call stack!", output
   end
 
+  class CallerLocation
+    attr_reader :path, :lineno, :label
+    alias_method :absolute_path, :path
+
+    def initialize(label, lineno)
+      @path = __FILE__
+      @lineno = lineno
+      @label = label
+    end
+  end
+
   test ":stderr behavior with #warn" do
     @deprecator.behavior = :stderr
 
     output = capture(:stderr) do
-      @deprecator.warn("Instance error!", ["instance call stack!"])
+      @deprecator.warn("Instance error!", [CallerLocation.new("instance call stack!", __LINE__)])
     end
 
     assert_match(/Instance error!/, output)
@@ -294,7 +301,7 @@ class DeprecationTest < ActiveSupport::TestCase
 
   test "DeprecatedInstanceVariableProxy" do
     instance = Deprecatee.new
-    instance.fubar = ActiveSupport::Deprecation::DeprecatedInstanceVariableProxy.new(instance, :foo_bar, "@fubar", @deprecator)
+    instance.fubar = ActiveSupport::Deprecation::DeprecatedInstanceVariableProxy.new(instance, :foo_bar, "@fubar", deprecator: @deprecator)
     instance.foo_bar = "foo bar!"
 
     fubar_size = assert_deprecated("@fubar.size", @deprecator) { instance.fubar.size }
@@ -306,15 +313,15 @@ class DeprecationTest < ActiveSupport::TestCase
 
   test "DeprecatedInstanceVariableProxy does not warn on inspect" do
     instance = Deprecatee.new
-    instance.fubar = ActiveSupport::Deprecation::DeprecatedInstanceVariableProxy.new(instance, :foo_bar, "@fubar", @deprecator)
+    instance.fubar = ActiveSupport::Deprecation::DeprecatedInstanceVariableProxy.new(instance, :foo_bar, "@fubar", deprecator: @deprecator)
     instance.foo_bar = "foo bar!"
 
     fubar_inspected = assert_not_deprecated(@deprecator) { instance.fubar.inspect }
     assert_equal instance.foo_bar.inspect, fubar_inspected
   end
 
-  test "DeprecatedInstanceVariableProxy without a deprecator is deprecated" do
-    assert_deprecated(ActiveSupport.deprecator) do
+  test "DeprecatedInstanceVariableProxy requires a deprecator" do
+    assert_raises(ArgumentError) do
       ActiveSupport::Deprecation::DeprecatedInstanceVariableProxy.new(Deprecatee.new, :foobar, "@fubar")
     end
   end
@@ -346,8 +353,8 @@ class DeprecationTest < ActiveSupport::TestCase
     end
   end
 
-  test "DeprecatedConstantProxy without a deprecator is deprecated" do
-    assert_deprecated(ActiveSupport.deprecator) do
+  test "DeprecatedConstantProxy requires a deprecator" do
+    assert_raise(ArgumentError) do
       ActiveSupport::Deprecation::DeprecatedConstantProxy.new("Fuu", "Undeprecated::Foo")
     end
   end
@@ -375,9 +382,9 @@ class DeprecationTest < ActiveSupport::TestCase
     end
   end
 
-  test "deprecate_constant is deprecated without a deprecator" do
+  test "deprecate_constant requires a deprecator" do
     legacy = Module.new.include(ActiveSupport::Deprecation::DeprecatedConstantAccessor)
-    assert_deprecated("DeprecatedConstantAccessor.deprecate_constant without a deprecator is deprecated", ActiveSupport.deprecator) do
+    assert_raises(ArgumentError) do
       legacy.deprecate_constant "OLD", "NEW"
     end
   end
@@ -770,180 +777,6 @@ class DeprecationTest < ActiveSupport::TestCase
     end
   end
 
-  test "warn delegator is deprecated" do
-    _, deprecations = collect_deprecations(ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.warn "foo"
-    end
-    assert_equal 2, deprecations.size
-    assert_match("foo", deprecations.first)
-    assert_match("use your own Deprecation object instead", deprecations.last)
-  end
-
-  test "deprecate_methods delegator is deprecated" do
-    assert_deprecated("use your own Deprecation object instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.deprecate_methods(Deprecatee)
-    end
-  end
-
-  test "silence delegator is deprecated" do
-    assert_deprecated("use Rails.application.deprecators.silence instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.silence { }
-    end
-  end
-
-  test "silence delegator is deprecated and delegates to the application's deprecators when available" do
-    with_rails_application_deprecators do
-      assert_deprecated("use Rails.application.deprecators.silence instead", ActiveSupport.deprecator) do
-        ActiveSupport::Deprecation.silence do
-          assert_not_deprecated(@deprecator) do
-            @deprecator.warn
-          end
-        end
-      end
-    end
-  end
-
-  test "allow delegator is deprecated" do
-    assert_deprecated("use Rails.application.deprecators[framework].allow", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.allow { }
-    end
-  end
-
-  test "behavior delegators are deprecated" do
-    old_behavior = ActiveSupport::Deprecation._instance.behavior
-    assert_deprecated("use Rails.application.deprecators[framework].behavior", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.behavior
-    end
-    assert_deprecated("use Rails.application.deprecators.behavior= instead", ActiveSupport.deprecator) do
-      # we have to keep the same behavior for assert_deprecated to work
-      ActiveSupport::Deprecation.behavior = ActiveSupport.deprecator.behavior
-    end
-  ensure
-    ActiveSupport::Deprecation._instance.behavior = old_behavior
-  end
-
-  test "behavior= delegator is deprecated and delegates to the application's deprecators when available" do
-    with_rails_application_deprecators do
-      called = false
-      assert_deprecated("use Rails.application.deprecators.behavior= instead", ActiveSupport.deprecator) do
-        ActiveSupport::Deprecation.behavior = ->(*) { called = true }
-        @deprecator.warn
-      end
-      assert called
-    end
-  end
-
-  test "disallowed_behavior delegators are deprecated" do
-    old_behavior = ActiveSupport::Deprecation._instance.disallowed_behavior
-    assert_deprecated("use Rails.application.deprecators[framework].disallowed_behavior", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.disallowed_behavior
-    end
-    assert_deprecated("use Rails.application.deprecators.disallowed_behavior= instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.disallowed_behavior = ->(*) { }
-    end
-  ensure
-    ActiveSupport::Deprecation._instance.disallowed_behavior = old_behavior
-  end
-
-  test "disallowed_behavior= delegators is deprecated and delegates to the application's deprecators when available" do
-    with_rails_application_deprecators do
-      called = false
-      assert_deprecated("use Rails.application.deprecators.disallowed_behavior= instead", ActiveSupport.deprecator) do
-        ActiveSupport::Deprecation.disallowed_behavior = ->(*) { called = true }
-      end
-      @deprecator.disallowed_warnings = :all
-      @deprecator.warn
-      assert called
-    end
-  end
-
-  test "debug delegators are deprecated" do
-    old_debug = ActiveSupport::Deprecation._instance.debug
-    assert_deprecated("use Rails.application.deprecators[framework].debug", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.debug
-    end
-    assert_deprecated("use Rails.application.deprecators.debug= instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.debug = true
-    end
-  ensure
-    ActiveSupport::Deprecation._instance.debug = old_debug
-  end
-
-  test "debug= delegator is deprecated and delegates to the application's deprecators when available" do
-    with_rails_application_deprecators do
-      assert_deprecated("use Rails.application.deprecators.debug= instead", ActiveSupport.deprecator) do
-        ActiveSupport::Deprecation.debug = true
-      end
-      assert @deprecator.debug
-    end
-  end
-
-  test "silenced delegators are deprecated" do
-    old_silenced = ActiveSupport::Deprecation._instance.silenced
-    assert_deprecated("use Rails.application.deprecators[framework].silenced", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.silenced
-    end
-    assert_deprecated("use Rails.application.deprecators.silenced= instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.silenced = false
-    end
-  ensure
-    ActiveSupport::Deprecation._instance.silenced = old_silenced
-  end
-
-  test "silenced= delegator is deprecated and delegates to the application's deprecators when available" do
-    with_rails_application_deprecators do
-      assert_deprecated("use Rails.application.deprecators.silenced= instead", ActiveSupport.deprecator) do
-        ActiveSupport::Deprecation.silenced = true
-      end
-      assert @deprecator.silenced
-    end
-  end
-
-  test "disallowed_warnings delegators are deprecated" do
-    old_disallowed_warnings = ActiveSupport::Deprecation._instance.disallowed_warnings
-    assert_deprecated("use Rails.application.deprecators[framework].disallowed_warnings", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.disallowed_warnings
-    end
-    assert_deprecated("use Rails.application.deprecators.disallowed_warnings= instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.disallowed_warnings = []
-    end
-  ensure
-    ActiveSupport::Deprecation._instance.disallowed_warnings = old_disallowed_warnings
-  end
-
-  test "disallowed_warnings= delegator is deprecated and delegates to the application's deprecators when available" do
-    with_rails_application_deprecators do
-      assert_deprecated("use Rails.application.deprecators.disallowed_warnings= instead", ActiveSupport.deprecator) do
-        ActiveSupport::Deprecation.disallowed_warnings = :all
-      end
-      assert_equal :all, @deprecator.disallowed_warnings
-    end
-  end
-
-  test "gem_name delegators are deprecated" do
-    old_gem_name = ActiveSupport::Deprecation._instance.gem_name
-    assert_deprecated("use your own Deprecation object instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.gem_name
-    end
-    assert_deprecated("use your own Deprecation object instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.gem_name = "MyGem"
-    end
-  ensure
-    ActiveSupport::Deprecation._instance.gem_name = old_gem_name
-  end
-
-  test "deprecation_horizon delegators are deprecated" do
-    old_deprecation_horizon = ActiveSupport::Deprecation._instance.deprecation_horizon
-    assert_deprecated("use your own Deprecation object instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.deprecation_horizon
-    end
-    assert_deprecated("use your own Deprecation object instead", ActiveSupport.deprecator) do
-      ActiveSupport::Deprecation.deprecation_horizon = "2.0"
-    end
-  ensure
-    ActiveSupport::Deprecation._instance.deprecation_horizon = old_deprecation_horizon
-  end
-
   test "warn deprecation skips the internal caller locations" do
     @deprecator.behavior = ->(_, callstack, *) { @callstack = callstack }
     method_that_emits_deprecation(@deprecator)
@@ -960,9 +793,26 @@ class DeprecationTest < ActiveSupport::TestCase
   test "warn deprecation can blame code generated with eval" do
     @deprecator.behavior = ->(message, *) { @message = message }
     generated_method_that_call_deprecation(@deprecator)
-    assert_match(%r{DEPRECATION WARNING: Here \(called from .*generated_method_that_call_deprecation at /path/to/template.html.erb:2\)}, @message)
+    if RUBY_VERSION >= "3.4"
+      assert_equal "DEPRECATION WARNING: Here (called from DeprecationTest#generated_method_that_call_deprecation at /path/to/template.html.erb:2)", @message
+    else
+      assert_equal "DEPRECATION WARNING: Here (called from generated_method_that_call_deprecation at /path/to/template.html.erb:2)", @message
+    end
   end
 
+  test "warn deprecation can blame code from internal methods" do
+    @deprecator.behavior = ->(message, *) { @message = message }
+    method_that_emits_deprecation_with_internal_method(@deprecator)
+
+    assert_includes(@message, "/path/to/user/code.rb")
+  end
+
+  class_eval(<<~RUBY, "/path/to/user/code.rb", 1)
+    def method_that_emits_deprecation_with_internal_method(deprecator)
+      [1].each { deprecator.warn }
+    end
+  RUBY
+
   private
     def method_that_emits_deprecation(deprecator)
       deprecator.warn
