@@ -54,7 +54,7 @@ class AssertionsTest < ActiveSupport::TestCase
         @object.increment
       end
     end
-    assert_equal "\"@object.num\" didn't change by 0, but by 1.\nExpected: 0\n  Actual: 1", error.message
+    assert_equal "`@object.num` didn't change by 0, but by 1.\nExpected: 0\n  Actual: 1", error.message
   end
 
   def test_assert_no_difference_with_message_fail
@@ -63,7 +63,7 @@ class AssertionsTest < ActiveSupport::TestCase
         @object.increment
       end
     end
-    assert_equal "Object Changed.\n\"@object.num\" didn't change by 0, but by 1.\nExpected: 0\n  Actual: 1", error.message
+    assert_equal "Object Changed.\n`@object.num` didn't change by 0, but by 1.\nExpected: 0\n  Actual: 1", error.message
   end
 
   def test_assert_no_difference_with_multiple_expressions_pass
@@ -157,7 +157,7 @@ class AssertionsTest < ActiveSupport::TestCase
         @object.increment
       end
     end
-    assert_equal "Object Changed.\n\"@object.num\" didn't change by 0, but by 1.\nExpected: 0\n  Actual: 1", error.message
+    assert_equal "Object Changed.\n`@object.num` didn't change by 0, but by 1.\nExpected: 0\n  Actual: 1", error.message
   end
 
   def test_assert_difference_message_includes_change
@@ -167,7 +167,17 @@ class AssertionsTest < ActiveSupport::TestCase
         @object.increment
       end
     end
-    assert_equal "\"@object.num\" didn't change by 5, but by 2.\nExpected: 5\n  Actual: 2", error.message
+    assert_equal "`@object.num` didn't change by 5, but by 2.\nExpected: 5\n  Actual: 2", error.message
+  end
+
+  def test_assert_difference_message_with_lambda
+    skip if !defined?(RubyVM::InstructionSequence)
+
+    error = assert_raises Minitest::Assertion do
+      assert_difference(-> { @object.num }, 1, "Object Changed") do
+      end
+    end
+    assert_equal "Object Changed.\n`@object.num` didn't change by 1, but by 0.\nExpected: 1\n  Actual: 0", error.message
   end
 
   def test_hash_of_lambda_expressions
@@ -233,7 +243,19 @@ class AssertionsTest < ActiveSupport::TestCase
       end
     end
 
-    assert_equal "\"@object.num\" didn't change. It was already 0.\nExpected 0 to not be equal to 0.", error.message
+    assert_equal "`@object.num` didn't change. It was already 0.\nExpected 0 to not be equal to 0.", error.message
+  end
+
+  def test_assert_changes_message_with_lambda
+    skip if !defined?(RubyVM::InstructionSequence)
+
+    error = assert_raises Minitest::Assertion do
+      assert_changes -> { @object.num }, to: 0 do
+        # no changes
+      end
+    end
+
+    assert_equal "`@object.num` didn't change. It was already 0.\nExpected 0 to not be equal to 0.", error.message
   end
 
   def test_assert_changes_with_wrong_to_option
@@ -331,7 +353,7 @@ class AssertionsTest < ActiveSupport::TestCase
         @object.increment
       end
     end
-    assert_equal "Expected initial value of nil", error.message
+    assert_equal "Expected initial value of nil, got 0", error.message
   end
 
   def test_assert_no_changes_with_from_and_case_operator
@@ -349,7 +371,91 @@ class AssertionsTest < ActiveSupport::TestCase
       end
     end
 
-    assert_equal "@object.num should not change.\n\"@object.num\" changed.\nExpected: 0\n  Actual: 1", error.message
+    assert_equal "@object.num should not change.\n`@object.num` changed.\nExpected: 0\n  Actual: 1", error.message
+  end
+
+  def test_assert_no_changes_message_with_lambda
+    skip if !defined?(RubyVM::InstructionSequence)
+
+    error = assert_raises Minitest::Assertion do
+      assert_no_changes -> { @object.num } do
+        @object.increment
+      end
+    end
+    assert_equal "`@object.num` changed.\nExpected: 0\n  Actual: 1", error.message
+
+    check = Proc.new {
+      @object.num
+    }
+    error = assert_raises Minitest::Assertion do
+      assert_no_changes check do
+        @object.increment
+      end
+    end
+    assert_equal "`@object.num` changed.\nExpected: 1\n  Actual: 2", error.message
+
+    check = lambda {
+      @object.num
+    }
+    error = assert_raises Minitest::Assertion do
+      assert_no_changes check do
+        @object.increment
+      end
+    end
+    assert_equal "`@object.num` changed.\nExpected: 2\n  Actual: 3", error.message
+
+    error = assert_raises Minitest::Assertion do
+      assert_no_changes -> { @object.num } do
+        @object.increment
+      end
+    end
+    assert_equal "`@object.num` changed.\nExpected: 3\n  Actual: 4", error.message
+
+    error = assert_raises Minitest::Assertion do
+      assert_no_changes ->(a = nil) { @object.num } do
+        @object.increment
+      end
+    end
+    assert_match(/#<Proc:0x.*changed/, error.message)
+  end
+
+  def test_assert_no_changes_message_with_multi_line_lambda
+    check = lambda {
+      "title".upcase
+      @object.num
+    }
+    error = assert_raises Minitest::Assertion do
+      assert_no_changes check do
+        @object.increment
+      end
+    end
+    assert_match(/#<Proc:0x.*changed/, error.message)
+
+    check = lambda {
+      "title".upcase
+      @object.num
+    }
+    error = assert_raises Minitest::Assertion do
+      assert_no_changes check do
+        @object.increment
+      end
+    end
+    assert_match(/#<Proc:0x.*changed/, error.message)
+  end
+
+  def test_assert_no_changes_message_with_not_real_callable
+    check = Object.new
+    def check.call
+      @object.num
+    end
+    check.instance_variable_set(:@object, @object)
+
+    error = assert_raises Minitest::Assertion do
+      assert_no_changes check do
+        @object.increment
+      end
+    end
+    assert_match(/#<Object:0x.*changed/, error.message)
   end
 
   def test_assert_no_changes_with_long_string_wont_output_everything
@@ -362,7 +468,7 @@ class AssertionsTest < ActiveSupport::TestCase
     end
 
     assert_match <<~output, error.message
-      "lines" changed.
+      `lines` changed.
       --- expected
       +++ actual
       @@ -10,4 +10,5 @@
@@ -403,7 +509,7 @@ class ExceptionsInsideAssertionsTest < ActiveSupport::TestCase
       run_test_that_should_fail_but_not_log_a_warning
     end
     assert_not @out.string.include?("assert_nothing_raised")
-    assert error.message.include?("(lambda)> changed")
+    assert error.message.include?("`rand` changed")
   end
 
   def test_fails_and_warning_is_logged_if_wrong_error_caught
@@ -417,7 +523,7 @@ class ExceptionsInsideAssertionsTest < ActiveSupport::TestCase
     MSG
     assert_includes @out.string, expected
     assert_includes error.message, "ArgumentError: ArgumentError"
-    assert_match(/in .*run_test_that_should_fail_confusingly'/, error.message)
+    assert_includes error.message, "run_test_that_should_fail_confusingly"
   end
 
   private
@@ -580,7 +686,7 @@ class TestConstStubbing < ActiveSupport::TestCase
     assert_equal 1, ConstStubbable::CONSTANT
   end
 
-  test "trying to stub a constant that does not exist in the receiver raises NameError" do
+  test "stubbing a constant that does not exist in the receiver raises NameError" do
     assert_raises(NameError) do
       stub_const(ConstStubbable, :NOT_A_CONSTANT, 1) { }
     end
@@ -589,4 +695,18 @@ class TestConstStubbing < ActiveSupport::TestCase
       stub_const(SubclassOfConstStubbable, :CONSTANT, 1) { }
     end
   end
+
+  test "stubbing a constant that does not exist can be done with `exists: false`" do
+    stub_const(ConstStubbable, :NOT_A_CONSTANT, 1, exists: false) do
+      assert_equal 1, ConstStubbable::NOT_A_CONSTANT
+    end
+
+    assert_raises(NameError) do
+      ConstStubbable::NOT_A_CONSTANT
+    end
+
+    assert_raises(NameError) do
+      stub_const(Object, :ConstStubbable, 1, exists: false)
+    end
+  end
 end
