@@ -5,8 +5,6 @@ require "bundler/inline"
 gemfile(true) do
   source "https://rubygems.org"
 
-  git_source(:github) { |repo| "https://github.com/#{repo}.git" }
-
   gem "rails"
   # If you want to test against edge Rails replace the previous line with this:
   # gem "rails", github: "rails/rails", branch: "main"
@@ -38,7 +36,7 @@ end
 require "minitest/autorun"
 require "rack/test"
 
-class BugTest < Minitest::Test
+class BugTest < ActiveSupport::TestCase
   include Rack::Test::Methods
 
   def test_returns_success
