@@ -182,8 +182,27 @@ module ApplicationTests
     end
 
     def test_code_statistics
-      assert_match "Code LOC: 62     Test LOC: 5     Code to Test Ratio: 1:0.1",
-        rails("stats")
+      assert_match(/Code LOC: \d+\s+Test LOC: \d+\s+ Code to Test Ratio: 1:\w+/, rails("stats"))
+    end
+
+    def test_reload_routes
+      add_to_config <<-RUBY
+        rake_tasks do
+          task do_something: :environment do
+            Rails.application.reload_routes!
+            puts Rails.application.routes.named_routes.to_h.keys.join(" ")
+          end
+        end
+      RUBY
+
+      app_file "config/routes.rb", <<-RUBY
+        Rails.application.routes.draw do
+          get "foo", to: "foo#bar", as: :my_great_route
+        end
+      RUBY
+
+      output = Dir.chdir(app_path) { `bin/rails do_something` }
+      assert_includes(output.lines.first, "my_great_route")
     end
 
     def test_loading_specific_fixtures
